from flask_wtf import FlaskForm
from wtforms import StringField, PasswordField, SubmitField,BooleanField
from wtforms.validators import DataRequired, Email, EqualTo, ValidationError
from flask_babel import _
from .models import User

input_text = "input-text"
input_submit= "submit-button"

class RegistrationForm(FlaskForm):
    fname = StringField(_('First Name'), validators=[DataRequired()],render_kw={"class": input_text})
    email = StringField(_('Email'), validators=[DataRequired(), Email()],render_kw={"class": input_text})
    password = PasswordField(_('Password'), validators=[DataRequired()],render_kw={"class": input_text})
    submit = SubmitField(_('Register'),render_kw={"class": input_submit})
    
    def validate_email(self, field):
        if User.query.filter_by(email=field.data).first():
            raise ValidationError(_('An account with this email already exists.'))


class RegistrationByInviteForm(FlaskForm):
    invite_code = StringField('',render_kw={"class": "hidden"})
    fname = StringField(_('First Name'), validators=[DataRequired()],render_kw={"class": input_text})
    password = PasswordField(_('Password'), validators=[DataRequired()],render_kw={"class": input_text})
    submit = SubmitField(_('Register'),render_kw={"class": input_submit})
    


class LoginForm(FlaskForm):
    email = StringField(_('Email'), validators=[DataRequired(), Email()],render_kw={"class": input_text})
    password = PasswordField(_('Password'), validators=[DataRequired()],render_kw={"class": input_text})
    remember = BooleanField('',default=True,render_kw={"class": "hidden"}) # _('Remember Me')
    submit = SubmitField(_('Log In'),render_kw={"class": input_submit})
    
class ResetPasswordForm(FlaskForm):
    password = PasswordField(_('New Password'), validators=[DataRequired()],render_kw={"class": input_text})
    confirm_password = PasswordField(_('Confirm Password'),
                                     validators=[DataRequired(), EqualTo('password')],render_kw={"class": input_submit})
    submit = SubmitField(_('Reset Password'),render_kw={"class": input_submit})
    
    
class RequestResetForm(FlaskForm):
    email = StringField(_('Email'), validators=[DataRequired(), Email()],render_kw={"class": input_text})
    submit = SubmitField(_('Request Password Reset'),render_kw={"class": input_submit})


class InviteForm(FlaskForm):
    email = StringField(_('Email'), validators=[DataRequired(), Email()],render_kw={"class": input_text})
    submit = SubmitField(_('Create Invite'),render_kw={"class": input_submit})



from flask_mail import Message,Mail
from flask import render_template, current_app
from . import mail


def send_email(to, subject, template, **kwargs):
    app = current_app._get_current_object()  # Get the actual application instance
    msg = Message(subject, recipients=[to], sender=app.config['MAIL_DEFAULT_SENDER'])
    msg.body = render_template(template + '.txt', **kwargs)
    msg.html = render_template(template + '.html', **kwargs)
    with app.app_context():
        mail.send(msg)



def get_locale():
  print('getting locale and the result is es')
  return 'es'
   # return request.accept_languages.best_match(Config['LANGUAGES'])
   
   
def add_babel_translation_directory(new_dir,app):
    
    app.config['BABEL_TRANSLATION_DIRECTORIES'] = 'translations;../boilersaas/src/boilersaas/translations'
    return
    
    # # Check if 'BABEL_TRANSLATION_DIRECTORIES' exists in app.config
    # if 'BABEL_TRANSLATION_DIRECTORIES' in app.config:
    #     # Split the current value into a list, remove empty entries
    #     current_dirs = [d for d in app.config['BABEL_TRANSLATION_DIRECTORIES'].split(';') if d]
    #     # Add the new directory if it's not already in the list
       
    #     if new_dir not in current_dirs:
    #         current_dirs.append(new_dir)
    #     # Update the app.config with the new semi-colon separated string
    #     app.config['BABEL_TRANSLATION_DIRECTORIES'] = ';'.join(current_dirs)
        
    # else:
    #     # If it doesn't exist, set it to the new directory
    #     app.config['BABEL_TRANSLATION_DIRECTORIES'] = new_dir



from flask_migrate import current
from .utils.db import db
from .types import UserType,UserConnectMethod
from flask_login import UserMixin
from flask import current_app as app
from datetime import datetime  
from itsdangerous.url_safe import URLSafeTimedSerializer as Serializer
from flask_dance.consumer.storage.sqla import OAuthConsumerMixin


class User(db.Model,UserMixin):
    __tablename__ = app.config.get('DB_TABLE_USERS')

    id = db.Column(db.Integer, primary_key=True)
    fname = db.Column(db.String(255), nullable=False)
    email = db.Column(db.String(255), unique=True, nullable=False)
    password = db.Column(db.String(255), nullable=False)  # Store hashed passwords
    type = db.Column(db.Enum(UserType), nullable=False)
    reg_date = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)
    last_login = db.Column(db.DateTime, nullable=True)
    lang = db.Column(db.String(2), nullable=False, default='en')
    connect_method = db.Column(db.Enum(UserConnectMethod), nullable=False)
    extra_fields = db.Column(db.JSON, nullable=True)
    
    def get_reset_token(self, expires_sec=1800):
        s = Serializer(app.config.get('SECRET_KEY'))
        token = s.dumps({'user_id': self.id})
        s.loads(token,max_age=expires_sec)
        return token

    @staticmethod
    def verify_reset_token(token):
        s = Serializer(app.config.get('SECRET_KEY'))
        try:
            user_id = s.loads(token)['user_id']
        except:
            return None
        return User.query.get(user_id)


class OAuth(OAuthConsumerMixin, db.Model):
    user_id = db.Column(db.Integer, db.ForeignKey(User.id))
    user = db.relationship(User)


class Invite(db.Model):
    __tablename__ = app.config.get('DB_TABLE_INVITES')
    id = db.Column(db.Integer, primary_key=True)
    email = db.Column(db.String(255), unique=True, nullable=False)
    invite_code = db.Column(db.String(255), unique=True, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False, default=datetime.utcnow)  

    def __repr__(self):
        return f'<Invite {self.email}>'





import string
from flask import render_template, flash, redirect, url_for,request, current_app as app
from flask_login import current_user, login_user, logout_user
from .utils.db import db
from .types import UserType, UserConnectMethod
from .utils.mail import send_email
from . import bp
from .models import User, Invite
from .forms import InviteForm, RegistrationByInviteForm, RegistrationForm,LoginForm,RequestResetForm,ResetPasswordForm
from werkzeug.security import generate_password_hash,check_password_hash
from flask_babel import _
import secrets

@app.context_processor
def inject_translations():
    return dict(_=_,page_title='TODO : page_title',app_name='TODO : app_name')

@bp.route('/user')
def user():
    return _('hello world!')

@bp.route('/register', methods=['GET', 'POST'])
def register():

    if current_user.is_authenticated:
        return redirect(url_for('main.dashboard')) 
    
    if not app.config.get('SIGNUP_OPTIONS')['allow_site']:
        flash(_('Registration is not currently allowed.'), 'error')
        return redirect(url_for('main.index'))

    form = RegistrationForm()
    if form.validate_on_submit():
         # Check if email already exists
        # user_exists = User.query.filter_by(email=form.email.data).first()
        # if user_exists:
        #     flash(_('An account with this email already exists.'), 'error')
        #     return redirect(url_for('users.register'))  # Adjust to your actual registration page route

        hashed_pwd = generate_password_hash(form.password.data, method=app.config.get('PWD_HASH_METHOD'), salt_length=app.config.get('PWD_SALT_LENGTH'))
        user = User(fname=form.fname.data, email=form.email.data, password=hashed_pwd,connect_method=UserConnectMethod.Site, type=UserType.User, lang='en')
        db.session.add(user)
        db.session.commit()
        login_user(user, remember=True)
        flash(_('Congratulations, you are now a registered user!'),'success')
        send_email(user.email, 'Welcome!', 'email/welcome', username=user.fname)

        return redirect(url_for('main.dashboard'))  # Adjust according to your app's structure
    return render_template('register.html', title=_('Register'), form=form,SIGNUP_OPTIONS=app.config.get('SIGNUP_OPTIONS'))


@bp.route('/invite/<invite_code>', methods=['GET', 'POST'])
def register_invite(invite_code):
    
    if current_user.is_authenticated:
        return redirect(url_for('main.dashboard')) 
    
    if not app.config.get('SIGNUP_OPTIONS')['allow_invite']:
        flash(_('Invites are not currently allowed.'), 'error')
        return redirect(url_for('main.index'))

    form = RegistrationByInviteForm()
      
    #invite_code = request.args.get('invite_code') or request.form.get('invite_code')
    if invite_code:
        form.invite_code.data = invite_code
       
        invite = Invite.query.filter_by(invite_code=invite_code).first()
        if not invite:
            flash(_('Invalid or expired invite code.'), 'error')
            return redirect(url_for('main.index'))
    else:
        flash(_('No invite code provided.'), 'error')
        return redirect(url_for('main.index'))
    
    if form.validate_on_submit():
        
        hashed_pwd = generate_password_hash(form.password.data, method=app.config.get('PWD_HASH_METHOD'), salt_length=app.config.get('PWD_SALT_LENGTH'))
        user = User(fname=form.fname.data, email=invite.email, password=hashed_pwd,connect_method=UserConnectMethod.Site, type=UserType.User, lang='en')
        db.session.add(user)
        db.session.commit()
        login_user(user, remember=True)
        flash(_('Congratulations, you are now a registered user!'),'success')
        send_email(user.email, 'Welcome!', 'email/welcome', username=user.fname)

        return redirect(url_for('main.dashboard'))  # Adjust according to your app's structure
    return render_template('register_invite.html', title=_('Register'), form=form,email=invite.email,invite_code=invite_code)



@bp.route('/login', methods=['GET', 'POST'])
def login():
   
    if current_user.is_authenticated:
        return redirect(url_for('main.dashboard')) 
    
    form = LoginForm()  # Assuming you have a LoginForm similar to RegistrationForm
    if form.validate_on_submit():
        # Assuming User model has an email field and a method to query by email
        user = User.query.filter_by(email=form.email.data).first()
        if user and check_password_hash(user.password, form.password.data):
            login_user(user, remember=form.remember.data)  # Log in the user
            next_page = request.args.get('next')
            return redirect(next_page) if next_page else redirect(url_for('main.dashboard'))  # Adjust according to your app's structure
        else:
            flash(_('Invalid email or password'),'error')
    return render_template('login.html', title=_('Log In'), form=form,SIGNUP_OPTIONS=app.config.get('SIGNUP_OPTIONS'))

@bp.route('/logout')
def logout():
    if not current_user.is_authenticated:
        return redirect(url_for('main.index'))  # Adjust according to your app's structure
    
    logout_user()  # Log out the current user
    return redirect(url_for('main.index'))  # Redirect to homepage or login page


@bp.route('/reset_password', methods=['GET', 'POST'])
def reset_request():
    if current_user.is_authenticated:
        return redirect(url_for('main.dashboard'))
    form = RequestResetForm()
    if form.validate_on_submit():
        user = User.query.filter_by(email=form.email.data).first()
        # Instead of flashing an error, flash a generic message whether or not the email is found
        flash(_('If an account with that email exists, you will receive an email with instructions to reset your password.'), 'info')
        
        # Proceed with sending the email only if the user is found
        if user:
            token = user.get_reset_token()
            send_email(user.email, 'Password Reset Request',
                       'email/reset_password', user=user, token=token)
        
        return redirect(url_for('users.reset_request'))
    return render_template('reset_request.html', title=_('Reset Password'), form=form)


@bp.route('/reset_password/<token>', methods=['GET', 'POST'])
def reset_token(token):
    if current_user.is_authenticated:
        return redirect(url_for('main.dashboard'))
    user = User.verify_reset_token(token)
    if user is None:
        flash(_('That is an invalid or expired token'), 'warning')
        return redirect(url_for('users.reset_request'))
    form = ResetPasswordForm()
    if form.validate_on_submit():
        hashed_pwd = generate_password_hash(form.password.data, method=app.config.get('PWD_HASH_METHOD'), salt_length=app.config.get('PWD_SALT_LENGTH'))
        user.password = hashed_pwd
        db.session.commit()
        flash(_('Your password has been updated! You are now able to log in'), 'success')
        return redirect(url_for('users.login'))
    return render_template('reset_request_with_token.html', title=_('Reset Password'), form=form)


@bp.route('/user/<int:user_id>')
def get_user(user_id):
    user = User.query.get_or_404(user_id)
    return {
        'user_id': user.user_id,
        'fname': user.fname,
        'user_email': user.user_email,
        # Do not return sensitive information like passwords
        'user_type': user.user_type.name,
        'user_reg_date': user.user_reg_date.isoformat(),
        'user_last_login': user.user_last_login.isoformat() if user.user_last_login else None,
        'user_lang': user.user_lang,
        'user_connect_method': user.user_connect_method.name,
        'user_extra_fields': user.user_extra_fields
    }




@bp.route('/admin/invites', methods=['GET', 'POST'])
def manage_invites():
   

    
    invites = Invite.query.all()
    for invite in invites:
        invite.url = invite_url = url_for('users.register_invite', invite_code=invite.invite_code)
    return render_template('invites.html', invites=invites)
  
  
@bp.route('/admin/invite_create', methods=['GET', 'POST'])
def invite_create():
    form = InviteForm()
    print(f"Method: {request.method}, Form Valid: {form.validate_on_submit()}")

    if form.validate_on_submit():
        email = form.email.data
        # Generate a unique invite code
        invite_code = ''.join(secrets.choice(string.ascii_uppercase + string.digits) for _ in range(6))
        invite = Invite(email=email, invite_code=invite_code)
        db.session.add(invite)
        db.session.commit()
        flash(_('Invite created successfully!'), 'success')
        return redirect(url_for('users.manage_invites'))
  
    
    
    return render_template('invite_create.html',form=form)


import datetime
from flask_dance.contrib.google import make_google_blueprint,google
# from flask_dance.contrib.facebook import make_facebook_blueprint,facebook
from .utils.db import db
from .models import OAuth
from flask_dance.consumer.storage.sqla import SQLAlchemyStorage
from flask_login import current_user
from flask import flash, redirect, url_for,current_app as app
from .models import User,UserConnectMethod,UserType
from flask_login import login_user
from sqlalchemy import String, and_
from flask_babel import _


# Setup Flask-Dance Google Blueprint
google_blueprint = make_google_blueprint(
    client_id=app.config.get('GOOGLE_CLIENT_ID'),
    client_secret=app.config.get('GOOGLE_CLIENT_SECRET'),
    scope=["openid","https://www.googleapis.com/auth/userinfo.profile", "https://www.googleapis.com/auth/userinfo.email"],
    redirect_to="google.after_authorize"
)



google_blueprint.storage = SQLAlchemyStorage(OAuth, db.session, user=current_user,user_required=False)



@google_blueprint.route('/google/after_authorize')
def after_authorize():
    if not app.config.get('SIGNUP_OPTIONS')['allow_google']:
        flash(_('Google registration is not currently allowed.'), 'error')
        return redirect(url_for('main.index'))
    


    if not google.authorized:
       return redirect(url_for('google.login'))
    
    
    resp = google.get("/oauth2/v2/userinfo")

        
    assert resp.ok, resp.text
    info = resp.json()
  
    user = User.query.filter_by(email=info["email"]).first()
    if not user:
        user = User(fname=info['given_name'], email=info["email"], password='0000',connect_method=UserConnectMethod.Google, type=UserType.User, lang=info['locale'])
        db.session.add(user)
        db.session.commit()
        
    
    login_user(user, remember=True)
    
    # Since we must update the user id in the OAuth table, we must find the OAuth record with the token
    # the comparison is kind of intensive here.
    # So let's narrow the fook down the search for when our app gets popular and we have a lot of new users
    five_minutes_ago_utc = datetime.datetime.utcnow() - datetime.timedelta(minutes=5)
    access_token = google.token.get('access_token')
    id_token = google.token.get('id_token')
    oauth = OAuth.query.filter(
    OAuth.created_at >= five_minutes_ago_utc,
    OAuth.user_id.is_(None),
    and_(
    OAuth.token.op('->>')('access_token') == access_token,
    OAuth.token.op('->>')('id_token') == id_token
    )
    ).first()
  
    if oauth:
        oauth.user_id = user.id
        db.session.commit()
    else:
        raise Exception(_("No matching OAuth record found."))

        
   
    return redirect(url_for('main.dashboard'))  





# Cleanup task for orphaned sessions
# Can happen since the session if often created before the user without some ACID transaction
def cleanup_orphaned_sessions():
    
    one_day_ago_utc = datetime.utcnow() - datetime.timedelta(days=1)
    
    orphaned_sessions = OAuth.query.filter(
        OAuth.user_id.is_(None),
        OAuth.created_at < one_day_ago_utc
    ).all()
    
    for session in orphaned_sessions:
        db.session.delete(session)
    db.session.commit()




from setuptools import setup, find_packages

setup(
    name="boilersaas",
    version="0.6.5",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    include_package_data=True,
      package_data={
        # Include templates, translations, and any other non-python files
      "boilersaas": [
            "templates/**/*.html",  # Recursively include all HTML files in templates directory
            "translations/**/*",  # Recursively include all files in translations directory
        ],
      },
    install_requires=[
        "Babel==2.14.0",
        "Flask==3.0.2",
        "flask-babel==4.0.0",
        "Flask-Dance==7.1.0",
        "Flask-Login==0.6.3",
        "Flask-Mail==0.9.1",
        "Flask-SQLAlchemy==3.1.1",
        "Flask-WTF==1.2.1",
        "itsdangerous==2.1.2",
        "python-dotenv==1.0.1",
        "requests==2.31.0",
        "SQLAlchemy==2.0.28",
        "Werkzeug==3.0.1",
        "WTForms==3.1.2",
        "email_validator==2.1.1" # WFForms dependency, not included in Flask-WTF
    ],
    author="DS",
    author_email="hello@davidstern.me",
    description="A boilerplate for flask SAAS apps, with social sign in, email confirmation, and more.",
    keywords="flask,boilerplate,saas,app",
    url="http://example.com/your_package",  # Optional
)



from flask import Blueprint

from flask_login import LoginManager,current_user
from flask_babel import Babel
from .utils.locale import add_babel_translation_directory, get_locale
from flask_dance.consumer.storage.sqla import SQLAlchemyStorage
from flask_dance.consumer import oauth_authorized
from flask_mail import Mail



login_manager = LoginManager()
login_manager.login_view = 'users.login'

bp = Blueprint('users', __name__,template_folder='templates')




@login_manager.user_loader
def load_user(user_id):
    from .models import User
    return User.query.get(int(user_id))

mail = None

def init_boilerplate_app(app):
    with app.app_context():
        from . import models,routes 
        from .signin_social import google_blueprint 

        app.register_blueprint(bp)
        app.register_blueprint(google_blueprint,url_prefix="/login")  
    
    
    add_babel_translation_directory('translations',app)
    babel = Babel(app)
    babel.init_app(app,locale_selector=get_locale)  
    
    
    login_manager.init_app(app) 
    
    global mail
    mail = Mail(app)
    
    @app.context_processor
    def inject_user_status():
        return dict(is_logged_in=current_user.is_authenticated)
    
    

    




import enum

class UserType(enum.Enum):
    Admin = 1
    User = 2
    Guest = 3

class UserConnectMethod(enum.Enum):
    Site = 'site'
    Google = 'google'
    GitHub = 'github'
    Facebook = 'fb'
    